//
// Test of G. Marsaglia's MWC256 PRNG.
//

#include <stdio.h>
#include <stdlib.h>

#define TRIALS 1000000


/* Implementation of a 32-bit KISS generator which uses no multiply instructions */
static unsigned int x = 123456789;
static unsigned int y = 234567891;
static unsigned int z = 345678912;
static unsigned int w = 456789123;
static unsigned int c = 0;

unsigned int JKISS32()
{
	int t;
	y ^= (y << 5);
	y ^= (y >> 7);
	y ^= (y << 22);
	t = z + w + c;
	z = w;
	c = t < 0;
	w = t & 2147483647;
	x += 1411392427;
	return (x + y + w);
}


/* MWC256 from Usenet posting by G. Marsaglia - Period 2^8222 */
static unsigned int Q[256], cc=362436;

unsigned int MWC256(void)
{
	unsigned long long t;
	static unsigned char i = 255;
	t = 809430660ULL * Q[++i] + cc;
	cc = (t >> 32);
	return (Q[i] = t);
}

void MCW256_seed()
{
	int i;
	x = 123456789;
	y = 234567891;
	z = 345678912;
	w = 456789123;
	c = 0;
	for (i = 0; i < 256; i++)
	{
		Q[i] = JKISS32();
	} 
}

int main (int argc, char* atgv[])
{
	unsigned int i;
	unsigned int random;

	MCW256_seed();

        for (i = 0; i < TRIALS; i++)
	{
		random = MWC256();
		printf ("%08x\n", random);
	}
	printf("\n");

	return(0);	
}
