'----------------------------------------------------------------------------------------
'
' File:    umath_mwc256.spin
'
' Author:  Michael Rychlik
'
' License: MIT (See end of file for details)
'
' Revision History:
'
' 1.0      08-01-2012  Initial version.
'
'----------------------------------------------------------------------------------------
'
' Provides unsigned arithmetic functions specifically for use with the MWC256 PRNG.
'
' mult : Multiplication of two 32 bit operands to produce a 64 bit result.
'
' dadd : Addition of two 64 bit operands to produce a 64 bit result.
'
' high : Returns the top 32 bits of the results of mult and dadd
'
' low  : Returns the low 32 bits of the results of mult or dadd
'
'
' This object is derived from Philip C. Pilgrim's umath object by:
' Removing all methods except mult, dadd, lt and cpr.
' Making mult and dadd PUBlic
' Adding methods high and low.
'
'----------------------------------------------------------------------------------------
CON
    UGT = %100
    EQ  = %010
    ULT = %001

VAR
    long  producth, productl

' 32 bit by 32 bit multiply with 64 bit result in producth and  productl
PUB mult(mplr, mpld) | umplr, umpld
    producth := (mplr & $7fff_ffff) ** (mpld & $7fff_ffff)
    productl := (mplr & $7fff_ffff) * (mpld & $7fff_ffff)
    if (mplr < 0)
        dadd(producth, productl, mpld >> 1, mpld << 31)
    if (mpld < 0)
        dadd(producth, productl, mplr << 1 >> 2, mplr << 31)

' 64 bit addition with result in producth and productl
PUB dadd(addh, addl, augh, augl)
    producth := addh + augh
    productl := addl + augl
    if (lt(productl, addl))
        producth++

' Get high part of mult or dadd result
PUB high
    return producth

' Get low part of mult or dadd result
PUB low
    return productl

' Return true if x < y unsigned.
PRI lt(x,y)
    return cpr(x,y) & ULT <> 0

' 64 bit unsigned compare
PRI cpr(x, y)
    if (x == y)
        return EQ
    elseif (x ^ $8000_0000 > y ^ $8000_0000)
        return UGT
    else
        return ULT

'----------------------------------------------------------------------------------------
'                         TERMS OF USE: MIT License
'----------------------------------------------------------------------------------------
' Permission is hereby granted, free of charge, to any person obtaining a copy of
' this software and associated documentation files (the "Software"), to deal in
' the Software without restriction, including without limitation the rights to
' use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
' the Software, and to permit persons to whom the Software is furnished to do so,
' subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all
' copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
' FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
' COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
' IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
' CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'----------------------------------------------------------------------------------------

